@@ -1,21 +1,31 @@
 
 
-save_question_submission <- function(session, label, question, answers, correct) {
+save_question_submission <- function(session, label, question, answer) {
   save_object(
-    session = session, 
-    object_id = label, 
+    session = session,
+    object_id = label,
+    tutorial_object("question_submission", list(
+      api_version = 1,
+      question = question,
+      answer = answer
+    ))
+  )
+}
+save_reset_question_submission <- function(session, label, question) {
+  save_object(
+    session = session,
+    object_id = label,
     tutorial_object("question_submission", list(
       question = question,
-      answers = answers,
-      correct = correct
+      reset = TRUE
     ))
   )
 }
 
 save_exercise_submission <- function(session, label, code, output, error_message, checked, feedback) {
-  
+
   # for client storage we only forward error output. this is because we want
-  # to replay errors back into the client with no execution (in case they were 
+  # to replay errors back into the client with no execution (in case they were
   # timeout errors as a result of misbehaving code). for other outputs the client
   # will just tickle the inputs to force re-execution of the outputs.
   storage <- tutorial_storage(session)
@@ -25,18 +35,18 @@ save_exercise_submission <- function(session, label, code, output, error_message
     else
       output <- NULL
   }
-   
+
   # save object
   save_object(
-    session = session, 
-    object_id = label, 
+    session = session,
+    object_id = label,
     tutorial_object("exercise_submission", list(
       code = code,
       output = output,
       checked = checked,
       feedback = feedback
     ))
-  )  
+  )
 }
 
 
@@ -50,8 +60,8 @@ save_section_skipped <- function(session, sectionId) {
 
 save_video_progress <- function(session, video_url, time, total_time) {
   save_object(
-    session = session, 
-    object_id = video_url, 
+    session = session,
+    object_id = video_url,
     tutorial_object("video_progress", list(
       time = time,
       total_time = total_time
@@ -83,18 +93,20 @@ get_exercise_submission <- function(session, label) {
 
 
 get_all_state_objects <- function(session, exercise_output = TRUE) {
-  
+
   # get all of the objects
   objects <- get_objects(session)
-  
+
   # strip output (the client doesn't need it and it's expensive to transmit)
   objects <- lapply(objects, function(object) {
-    if (object$type == "exercise_submission")
-      if (!exercise_output)
+    if (object$type == "exercise_submission") {
+      if (!exercise_output) {
         object$data["output"] <- list(NULL)
+      }
+    }
     object
   })
-  
+
   # return objects
   objects
 }
@@ -106,7 +118,15 @@ filter_state_objects <- function(state_objects, types) {
 }
 
 submissions_from_state_objects <- function(state_objects) {
-  filter_state_objects(state_objects, c("question_submission", "exercise_submission"))
+  filtered_submissions <- filter_state_objects(state_objects, c("question_submission", "exercise_submission"))
+  Filter(x = filtered_submissions, function(object) {
+    # only return answered question, not reset questions
+    if (object$type == "question_submission") {
+      !isTRUE(object$data$reset)
+    } else {
+      TRUE
+    }
+  })
 }
 
 video_progress_from_state_objects <- function(state_objects) {
@@ -123,23 +143,24 @@ progress_events_from_state_objects <- function(state_objects) {
   # first submissions
   submissions <- submissions_from_state_objects(state_objects)
   progress_events <- lapply(submissions, function(submission) {
+    data <- list(
+      label = submission$id
+    )
     if (submission$type == "question_submission") {
-      correct <- submission$data$correct
+      data$answer <- submission$data$answer
     }
     else if (submission$type == "exercise_submission") {
       if (!is.null(submission$data$feedback))
         correct <- submission$data$feedback$correct
       else
         correct <- TRUE
+      data$correct <- correct
     }
-    
+
     list(event = submission$type,
-         data = list(
-           label = submission$id, 
-           correct = correct
-         ))
+         data = data)
   })
-  
+
   # now sections skipped
   section_skipped_progress <- section_skipped_progress_from_state_objects(state_objects)
   section_skipped_progress_events <- lapply(section_skipped_progress, function(skipped) {
@@ -149,7 +170,7 @@ progress_events_from_state_objects <- function(state_objects) {
          ))
   })
   progress_events <- append(progress_events, section_skipped_progress_events)
-  
+
   # now video_progress
   video_progress <- video_progress_from_state_objects(state_objects)
   video_progress_events <- lapply(video_progress, function(progress) {
@@ -161,7 +182,7 @@ progress_events_from_state_objects <- function(state_objects) {
          ))
   })
   progress_events <- append(progress_events, video_progress_events)
-  
+
   # return progress events
   progress_events
 }
@@ -174,18 +195,44 @@ save_object <- function(session, object_id, data) {
   tutorial_storage(session)$save_object(tutorial_id, tutorial_version, user_id, object_id, data)
 }
 
+
+update_object <- function(object) {
+  if (is.null(object)) {
+    return(object)
+  }
+  if (identical(object$type, "question_submission")) {
+    api_version <- object$data$api_version
+    if (!is.null(api_version)) {
+      # if (identical(version, 1)) {
+      #   # do nothing
+      # }
+    } else {
+      # as of v0.10.0...
+      # upgrade from old storage format to new storage format
+      # rename answers -> answer
+      object$data$answer <- object$data$answers
+      object$data$answers <- NULL
+      # do not record correct information
+      object$data$correct <- NULL
+    }
+  }
+  object
+}
+
 get_object <- function(session, object_id) {
   tutorial_id <- read_request(session, "tutorial.tutorial_id")
   tutorial_version <- read_request(session, "tutorial.tutorial_version")
   user_id <- read_request(session, "tutorial.user_id")
-  tutorial_storage(session)$get_object(tutorial_id, tutorial_version, user_id, object_id)
+  object <- tutorial_storage(session)$get_object(tutorial_id, tutorial_version, user_id, object_id)
+  update_object(object)
 }
 
 get_objects <- function(session) {
   tutorial_id <- read_request(session, "tutorial.tutorial_id")
   tutorial_version <- read_request(session, "tutorial.tutorial_version")
   user_id <- read_request(session, "tutorial.user_id")
-  tutorial_storage(session)$get_objects(tutorial_id, tutorial_version, user_id)
+  objects <- tutorial_storage(session)$get_objects(tutorial_id, tutorial_version, user_id)
+  lapply(objects, update_object)
 }
 
 remove_all_objects <- function(session) {
@@ -224,12 +271,12 @@ ns_unwrap <- function(ns, id) {
 
 # get the currently active storage handler
 tutorial_storage <- function(session) {
-  
+
   # local storage implementation
   local_storage <- filesystem_storage(
     file.path(rappdirs::user_data_dir(), "R", "learnr", "tutorial", "storage")
   )
-  
+
   # function to determine "auto" storage
   auto_storage <- function() {
     location <- read_request(session, "tutorial.http_location")
@@ -238,14 +285,14 @@ tutorial_storage <- function(session) {
     else
       client_storage(session)
   }
-  
+
   # examine the option
   storage <- getOption("tutorial.storage", default = "auto")
 
   # resolve NULL to "none"
   if (is.null(storage))
     storage <- "none"
-  
+
   # if it's a character vector then resolve it
   if (is.character(storage)) {
     storage <- switch(storage,
@@ -255,12 +302,12 @@ tutorial_storage <- function(session) {
       none = no_storage()
     )
   }
-  
+
   # verify that storage is a list
   if (!is.list(storage))
-    stop("tutorial.storage must be a 'auto', 'local', 'client', 'none' or a ", 
+    stop("tutorial.storage must be a 'auto', 'local', 'client', 'none' or a ",
          "list of storage functions")
-  
+
   # validate storage interface
   if (is.null(storage$save_object))
     stop("tutorial.storage must implement the save_object function")
@@ -268,27 +315,27 @@ tutorial_storage <- function(session) {
     stop("tutorial.storage must implement the get_object function")
   if (is.null(storage$get_objects))
     stop("tutorial.storage must implements the get_objects function")
-  
+
   # return it
   storage
 }
 
 
 #' Filesystem-based storage for tutor state data
-#' 
+#'
 #' Tutorial state storage handler that uses the filesystem
-#' as a backing store. The direcotry will contain tutorial
-#' state data partitioned by user_id, tutorial_id, and 
+#' as a backing store. The directory will contain tutorial
+#' state data partitioned by user_id, tutorial_id, and
 #' tutorial_version (in that order)
-#' 
+#'
 #' @param dir Directory to store state data within
 #' @param compress Should \code{.rds} files be compressed?
-#' 
+#'
 #' @return Storage handler suitable for \code{options(tutorial.storage = ...)}
-#' 
+#'
 #' @export
 filesystem_storage <- function(dir, compress = TRUE) {
-  
+
   # helpers to transform ids into valid filesystem paths
   id_to_filesystem_path <- function(id) {
     id <- gsub("..", "", id, fixed = TRUE)
@@ -297,10 +344,10 @@ filesystem_storage <- function(dir, compress = TRUE) {
   id_from_filesystem_path <- function(path) {
     utils::URLdecode(path)
   }
-  
+
   # get the path to storage (ensuring that the directory exists)
   storage_path <- function(tutorial_id, tutorial_version, user_id) {
-    path <- file.path(dir, 
+    path <- file.path(dir,
                       id_to_filesystem_path(user_id),
                       id_to_filesystem_path(tutorial_id),
                       id_to_filesystem_path(tutorial_version))
@@ -308,27 +355,27 @@ filesystem_storage <- function(dir, compress = TRUE) {
       dir.create(path, recursive = TRUE)
     path
   }
-  
+
   # functions which implement storage via saving to RDS
   list(
-    
+
     type = "local",
-    
+
     save_object = function(tutorial_id, tutorial_version, user_id, object_id, data) {
-      object_path <- file.path(storage_path(tutorial_id, tutorial_version, user_id), 
+      object_path <- file.path(storage_path(tutorial_id, tutorial_version, user_id),
                                paste0(id_to_filesystem_path(object_id), ".rds"))
       saveRDS(data, file = object_path, compress = compress)
     },
-    
+
     get_object = function(tutorial_id, tutorial_version, user_id, object_id) {
-      object_path <- file.path(storage_path(tutorial_id, tutorial_version, user_id), 
+      object_path <- file.path(storage_path(tutorial_id, tutorial_version, user_id),
                                paste0(id_to_filesystem_path(object_id), ".rds"))
       if (file.exists(object_path))
         readRDS(object_path)
       else
         NULL
     },
-    
+
     get_objects = function(tutorial_id, tutorial_version, user_id) {
       objects_path <- storage_path(tutorial_id, tutorial_version, user_id)
       objects <- list()
@@ -339,57 +386,57 @@ filesystem_storage <- function(dir, compress = TRUE) {
       }
       objects
     },
-    
+
     remove_all_objects = function(tutorial_id, tutorial_version, user_id) {
       objects_path <- storage_path(tutorial_id, tutorial_version, user_id)
       unlink(objects_path, recursive = TRUE)
     }
-  ) 
+  )
 }
 
 # client side storage implementation. data is saved by broadcasting it to the client
 # this data is subsequently restored during initialize and stored in a per-session
 # in memory table for retreival
 client_storage <- function(session) {
-  
-  
+
+
   # helper to form a unique tutorial context id (note that we don't utilize the user_id
-  # as there is no concept of server-side user in client_storage, user scope is 100% 
+  # as there is no concept of server-side user in client_storage, user scope is 100%
   # determined by connecting user agent)
   tutorial_context_id <- function(tutorial_id, tutorial_version) {
     paste(tutorial_id, tutorial_version, sep = "-")
   }
-  
+
   # get a reference to the session object cache for a gvien tutorial context
   object_store <- function(context_id) {
-    
+
     # create session objects on demand
     session_objects <- read_request(session, "tutorial.session_objects")
     if (is.null(session_objects)) {
       session_objects <- new.env(parent = emptyenv())
       write_request(session, "tutorial.session_objects", session_objects)
     }
-    
+
     # create entry for this context on demand
     if (!exists(context_id, envir = session_objects))
       assign(context_id, new.env(parent = emptyenv()), envir = session_objects)
     store <- get(context_id, envir = session_objects)
-    
+
     # return reference to the store
     store
   }
-  
+
   list(
-    
+
     type = "client",
-  
+
     save_object = function(tutorial_id, tutorial_version, user_id, object_id, data) {
-      
+
       # save the object to our in-memory store
       context_id <- tutorial_context_id(tutorial_id, tutorial_version)
       store <- object_store(context_id)
       assign(object_id, data, envir = store)
-     
+
       # broadcast to client
       session$sendCustomMessage("tutorial.store_object", list(
         context = context_id,
@@ -397,7 +444,7 @@ client_storage <- function(session) {
         data = base64_enc(serialize(data, connection = NULL))
       ))
     },
-    
+
     get_object = function(tutorial_id, tutorial_version, user_id, object_id) {
       context_id <- tutorial_context_id(tutorial_id, tutorial_version)
       store <- object_store(context_id)
@@ -406,8 +453,8 @@ client_storage <- function(session) {
       else
         NULL
     },
-    
-    get_objects = function(tutorial_id, tutorial_version, user_id) { 
+
+    get_objects = function(tutorial_id, tutorial_version, user_id) {
       context_id <- tutorial_context_id(tutorial_id, tutorial_version)
       store <- object_store(context_id)
       objects <- list()
@@ -415,14 +462,14 @@ client_storage <- function(session) {
         objects[[length(objects) + 1]] <- get(object, envir = store)
       objects
     },
-    
+
     remove_all_objects = function(tutorial_id, tutorial_version, user_id) {
       # remove on server side (client side is handled on client)
       context_id <- tutorial_context_id(tutorial_id, tutorial_version)
       store <- object_store(context_id)
       rm(list = ls(store), envir = store)
     },
-    
+
     # function called from initialize to prime object storage from the browser db
     initialize_objects_from_client = function(tutorial_id, tutorial_version, user_id, objects) {
       context_id <- tutorial_context_id(tutorial_id, tutorial_version)
@@ -446,6 +493,3 @@ no_storage <- function() {
     remove_all_objects = function(tutorial_id, tutorial_version, user_id) {}
   )
 }
-
-
-
